@@ -12,7 +12,7 @@ from trytond.tests.test_tryton import doctest_checker
 class AccountTestCase(ModuleTestCase):
     'Test Account Es module'
     module = 'account_es'
-    extras = ['account_asset', 'account_payment_sepa']
+    extras = ['account_asset', 'account_payment_sepa', 'sale_advance_payment']
 
 
 def suite():
@@ -42,4 +42,9 @@ def suite():
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_reporting_surcharge_tax.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     return suite
